#!/usr/bin/env node
"use strict";
/// <reference types="./typing" />
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.localStorage = exports.vorpal = void 0;
const vorpal_1 = __importDefault(require("@moleculer/vorpal"));
const ms_1 = __importDefault(require("ms"));
const utils_1 = require("./src/utils");
const queue_1 = require("./src/queue");
exports.vorpal = new vorpal_1.default();
exports.vorpal.localStorage("bull-repl-default");
exports.localStorage = exports.vorpal.localStorage;
exports.vorpal
    .command("connect <queue>", "Connect to bull queue")
    .option("--prefix <prefix>", "Prefix to use for all queue jobs")
    .option("-u, --uri <uri>", "Redis uri connection string")
    .option("-h, --host <host>", "Redis host for connection")
    .option("-p, --port <port>", "Redis port for connection")
    .option("-d, --db <db>", "Redis db for connection")
    .option("--password <password>", "Redis password for connection")
    .option("-c, --cert <cert>", "Absolute path to pem certificate if TLS used")
    .option("-e, --exec <exec>", "Exec command")
    .option("-f, --execFile <execFile>", "Exec commands from file")
    .action((0, utils_1.wrapTryCatch)(async (params) => {
    await (0, queue_1.connectToQueue)(params, exports.vorpal);
    if (params.options.exec) {
        process.nextTick(async () => {
            await exports.vorpal.exec(params.options.exec);
            await exports.vorpal.exec('exit');
        });
    }
    if (params.options.execFile) {
        process.nextTick(async () => {
            const lines = (0, utils_1.readLines)(params.options.execFile);
            for await (const line of lines) {
                await (0, utils_1.logGreen)(line);
                await exports.vorpal.exec(line);
            }
            await exports.vorpal.exec('exit');
        });
    }
}));
exports.vorpal.command("connect-list", "List of saved connections").action((0, utils_1.wrapTryCatch)(async () => {
    console.table(exports.localStorage._localStorage._keys);
}));
exports.vorpal.command("connect-rm <name>", "Remove saved connection").action((0, utils_1.wrapTryCatch)(async ({ name }) => {
    if (name === utils_1.LAST_SAVED_CONNECTION_NAME) {
        return (0, utils_1.logYellow)(`Can't use reserved name, please use another`);
    }
    const savedItem = exports.localStorage.getItem(name);
    if (savedItem) {
        exports.localStorage.removeItem(name);
        (0, utils_1.logGreen)(`Connection "${name}" removed`);
    }
    else {
        (0, utils_1.logYellow)(`Connection "${name}" not found`);
    }
}));
exports.vorpal.command("connect-save <name>", "Save current connection").action((0, utils_1.wrapTryCatch)(async ({ name: nameForSave }) => {
    if (nameForSave === utils_1.LAST_SAVED_CONNECTION_NAME) {
        return (0, utils_1.logYellow)(`Can't use reserved name, please use another`);
    }
    await (0, queue_1.getQueue)();
    const options = JSON.parse(exports.localStorage.getItem(utils_1.LAST_SAVED_CONNECTION_NAME));
    exports.localStorage.setItem(nameForSave, JSON.stringify(options));
    (0, utils_1.logGreen)(`Connection "${nameForSave}" saved`);
}));
exports.vorpal.command("connect-to <name>", "Connect to saved connection").action((0, utils_1.wrapTryCatch)(async ({ name: connectToName }) => {
    const savedItem = exports.localStorage.getItem(connectToName);
    if (!savedItem) {
        return (0, utils_1.logYellow)(`Connection "${connectToName}" not found`);
    }
    const options = JSON.parse(savedItem);
    await (0, queue_1.connectToQueue)(options, exports.vorpal);
}));
exports.vorpal.command("stats", "Count of jobs by type").action((0, utils_1.wrapTryCatch)(async () => {
    const queue = await (0, queue_1.getQueue)();
    const counts = await queue.getJobCounts("completed", "failed", "delayed", "repeat", "active", "wait", "paused");
    console.table(counts);
}));
exports.vorpal
    .command("active", "Fetch active jobs")
    .option("-q, --query <query>", `Query jobs via jq - ${utils_1.jqLink}. Notice, that bull data in root key e.g '[.root[] | select(.progress > 70)]'`)
    .option("-t, --timeAgo <timeAgo>", `Get jobs since time ago via ${utils_1.msLink}`)
    .option("-s, --start <start>", "Start index (pagination)")
    .option("-e, --end <end>", "End index (pagination)")
    .action((0, utils_1.wrapTryCatch)(async ({ options }) => {
    const queue = await (0, queue_1.getQueue)();
    const timeAgoFilter = await (0, utils_1.getTimeAgoFilter)(options.timeAgo);
    (0, utils_1.showJobs)(await queue.getActive(options.start || 0, options.end || 100), [timeAgoFilter, options.query].filter(v => v).join(' | '));
}));
exports.vorpal
    .command("waiting", "Fetch waiting jobs")
    .option("-q, --query <query>", `Query jobs via jq - ${utils_1.jqLink}. Notice, that bull data in root key e.g '[.root[] | select(.progress > 70)]'`)
    .option("-t, --timeAgo <timeAgo>", `Get jobs since time ago via ${utils_1.msLink}`)
    .option("-s, --start <start>", "Start index (pagination)")
    .option("-e, --end <end>", "End index (pagination)")
    .action((0, utils_1.wrapTryCatch)(async ({ options }) => {
    const queue = await (0, queue_1.getQueue)();
    const timeAgoFilter = await (0, utils_1.getTimeAgoFilter)(options.timeAgo);
    (0, utils_1.showJobs)(await queue.getWaiting(options.start || 0, options.end || 100), [timeAgoFilter, options.query].filter(v => v).join(' | '));
}));
exports.vorpal
    .command("completed", "Fetch completed jobs")
    .option("-q, --query <query>", `Query jobs via jq - ${utils_1.jqLink}. Notice, that bull data in root key e.g '[.root[] | select(.progress > 70)]'`)
    .option("-t, --timeAgo <timeAgo>", `Get jobs since time ago via ${utils_1.msLink}`)
    .option("-s, --start <start>", "Start index (pagination)")
    .option("-e, --end <end>", "End index (pagination)")
    .action((0, utils_1.wrapTryCatch)(async ({ options }) => {
    const queue = await (0, queue_1.getQueue)();
    const timeAgoFilter = await (0, utils_1.getTimeAgoFilter)(options.timeAgo);
    (0, utils_1.showJobs)(await queue.getCompleted(options.start || 0, options.end || 100), [timeAgoFilter, options.query].filter(v => v).join(' | '));
}));
exports.vorpal
    .command("failed", "Fetch failed jobs")
    .option("-q, --query <query>", `Query jobs via jq - ${utils_1.jqLink}. Notice, that bull data in root key e.g '[.root[] | select(.progress > 70)]'`)
    .option("-t, --timeAgo <timeAgo>", `Get jobs since time ago via ${utils_1.msLink}`)
    .option("-s, --start <start>", "Start index (pagination)")
    .option("-e, --end <end>", "End index (pagination)")
    .action((0, utils_1.wrapTryCatch)(async ({ options }) => {
    const queue = await (0, queue_1.getQueue)();
    const timeAgoFilter = await (0, utils_1.getTimeAgoFilter)(options.timeAgo);
    (0, utils_1.showJobs)(await queue.getFailed(options.start || 0, options.end || 100), [timeAgoFilter, options.query].filter(v => v).join(' | '));
}));
exports.vorpal
    .command("delayed", "Fetch delayed jobs")
    .option("-q, --query <query>", `Query jobs via jq - ${utils_1.jqLink}. Notice, that bull data in root key e.g '[.root[] | select(.progress > 70)]'`)
    .option("-t, --timeAgo <timeAgo>", `get jobs since time ago via ${utils_1.msLink}`)
    .option("-s, --start <start>", "start index (pagination)")
    .option("-e, --end <end>", "end index (pagination)")
    .action((0, utils_1.wrapTryCatch)(async ({ options }) => {
    const queue = await (0, queue_1.getQueue)();
    const timeAgoFilter = await (0, utils_1.getTimeAgoFilter)(options.timeAgo);
    (0, utils_1.showJobs)(await queue.getDelayed(options.start || 0, options.end || 100), [timeAgoFilter, options.query].filter(v => v).join(' | '));
}));
exports.vorpal
    .command("pause", "Pause current queue")
    .option("-y, --yes", "Skip answer validation")
    .action((0, utils_1.wrapTryCatch)(async ({ options }) => {
    const queue = await (0, queue_1.getQueue)();
    await (0, utils_1.answer)(exports.vorpal, "Pause queue", options.yes);
    await queue.pause();
    (0, utils_1.logGreen)(`Queue paused`);
}));
exports.vorpal
    .command("resume", "Resume current queue from pause")
    .option("-y, --yes", "Skip answer validation")
    .action((0, utils_1.wrapTryCatch)(async ({ options }) => {
    const queue = await (0, queue_1.getQueue)();
    await (0, utils_1.answer)(exports.vorpal, "Resume queue", options.yes);
    await queue.resume();
    (0, utils_1.logGreen)(`Queue resumed from pause`);
}));
exports.vorpal.command("get <jobId...>", "Get job").action((0, utils_1.wrapTryCatch)(async ({ jobId }) => {
    const { notFoundIds, foundJobs } = await (0, utils_1.splitJobsByFound)(jobId);
    notFoundIds.length && (0, utils_1.logYellow)(`Not found jobs: ${notFoundIds}`);
    foundJobs.length && (0, utils_1.showJobs)(foundJobs, '');
}));
exports.vorpal
    .command("add <data>", "Add job to queue e.g. add '{\"x\": 1}'")
    .option("-n, --name <name>", "name for named job")
    .option("--jobId <jobId>", "Override the job ID - by default")
    .option("--priority <priority>", "Optional priority value. ranges from 1 (highest priority) to MAX_INT  (lowest priority)")
    .option("--delay <delay>", "An amount of milliseconds to wait until this job can be processed")
    .option("--attempts <attempts>", "The total number of attempts to try the job until it completes")
    .option("--repeat <repeat>", "Repeat job according to a cron specificatio")
    .option("--lifo <lifo>", "if true, adds the job to the right of the queue instead of the left (default false)")
    .option("-y, --yes", "Skip answer validation")
    .action((0, utils_1.wrapTryCatch)(async function ({ data, options }) {
    const queue = await (0, queue_1.getQueue)();
    const { priority, repeat, jobId, delay = 0, attempts = 1, lifo = false, } = options;
    let jobData;
    let jobOptions = Object.fromEntries(Object.entries({
        jobId,
        priority,
        repeat,
        delay,
        attempts,
        lifo,
    })
        .filter(([, value]) => value != null));
    try {
        jobData = JSON.parse(data);
    }
    catch (e) {
        return (0, utils_1.throwYellow)(`Error: Argument <data> is invalid: ${e}`);
    }
    if (repeat && typeof repeat === 'string') {
        try {
            jobOptions.repeat = JSON.parse(repeat);
        }
        catch (e) {
            return (0, utils_1.throwYellow)(`Error: Option --repeat is invalid: ${e}`);
        }
    }
    await (0, utils_1.answer)(exports.vorpal, "Add", options.yes);
    const jobName = options.name || "__default__";
    const addedJob = await queue.add(jobName, jobData, {
        ...jobOptions,
        timestamp: Date.now()
    });
    (0, utils_1.logGreen)(`Job with name '${jobName}', id '${addedJob.id}' added`);
}));
exports.vorpal
    .command("rm <jobId...>", "Remove job")
    .option("-y, --yes", "Skip answer validation")
    .action((0, utils_1.wrapTryCatch)(async function ({ jobId, options }) {
    await (0, utils_1.answer)(exports.vorpal, "Remove", options.yes);
    const { notFoundIds, foundJobs } = await (0, utils_1.splitJobsByFound)(jobId);
    await Promise.all(foundJobs.map(j => j.remove()));
    notFoundIds.length && (0, utils_1.logYellow)(`Not found jobs: ${notFoundIds}`);
    foundJobs.length && (0, utils_1.logGreen)(`Jobs "${foundJobs.map(j => j.id)}" removed`);
}));
exports.vorpal
    .command("retry <jobId...>", "Retry job")
    .option("-y, --yes", "Skip answer validation")
    .action((0, utils_1.wrapTryCatch)(async function ({ jobId, options }) {
    await (0, utils_1.answer)(exports.vorpal, "Retry", options.yes);
    const { notFoundIds, foundJobs } = await (0, utils_1.splitJobsByFound)(jobId);
    await Promise.all(foundJobs.map(j => j.retry()));
    notFoundIds.length && (0, utils_1.logYellow)(`Not found jobs: ${notFoundIds}`);
    foundJobs.length && (0, utils_1.logGreen)(`Jobs "${foundJobs.map(j => j.id)}" retried`);
}));
exports.vorpal
    .command("retry-failed", "Retry first 100 failed jobs")
    .option("-n, --number <number>", "Number of failed jobs. default: 100")
    .option("-y, --yes", "Skip answer validation")
    .action((0, utils_1.wrapTryCatch)(async function ({ options }) {
    const queue = await (0, queue_1.getQueue)();
    await (0, utils_1.answer)(exports.vorpal, "Retry failed jobs", options.yes);
    const failedJobs = await queue.getFailed(0, options.number || 100);
    await Promise.all(failedJobs.map(j => j.retry()));
    (0, utils_1.logGreen)("All failed jobs retried");
}));
exports.vorpal
    .command("promote <jobId...>", "Promote job")
    .option("-y, --yes", "Skip answer validation")
    .action((0, utils_1.wrapTryCatch)(async function ({ jobId, options }) {
    await (0, utils_1.answer)(exports.vorpal, "Promote", options.yes);
    const { notFoundIds, foundJobs } = await (0, utils_1.splitJobsByFound)(jobId);
    await Promise.all(foundJobs.map(j => j.promote()));
    notFoundIds.length && (0, utils_1.logYellow)(`Not found jobs: ${notFoundIds}`);
    foundJobs.length && (0, utils_1.logGreen)(`Jobs "${foundJobs.map(j => j.id)}" promoted`);
}));
exports.vorpal
    .command("fail <jobId> <reason>", "Move job to failed")
    .option("-y, --yes", "Skip answer validation")
    .action((0, utils_1.wrapTryCatch)(async function ({ jobId, reason, options }) {
    await (0, queue_1.getQueue)();
    const job = await (0, utils_1.getJob)(jobId);
    await (0, utils_1.answer)(exports.vorpal, "Fail", options.yes);
    const err = new Error(reason);
    await job.moveToFailed(err, "0");
    (0, utils_1.logGreen)(`Job "${jobId}" failed`);
}));
exports.vorpal
    .command("complete <jobId> <data>", "Move job to completed e.g. complete 1 '{\"x\": 1}'")
    .option("-y, --yes", "Skip answer validation")
    .action((0, utils_1.wrapTryCatch)(async function ({ jobId, data, options }) {
    await (0, queue_1.getQueue)();
    const job = await (0, utils_1.getJob)(jobId);
    let returnValue;
    try {
        returnValue = JSON.parse(data);
    }
    catch (e) {
        return (0, utils_1.throwYellow)(`Error: Argument <data> is invalid: ${e}`);
    }
    await (0, utils_1.answer)(exports.vorpal, "Complete", options.yes);
    await job.moveToCompleted(returnValue, "0");
    (0, utils_1.logGreen)(`Job "${jobId}" completed`);
}));
exports.vorpal
    .command("clean <period>", `Clean queue for period ago, period format - ${utils_1.msLink}`)
    .option("-s, --status <status>", "Status of the job to clean, default: completed")
    .option("-l, --limit <limit>", "Maximum amount of jobs to clean per call, default: all")
    .option("-y, --yes", "Skip answer validation")
    .action((0, utils_1.wrapTryCatch)(async function ({ period, options }) {
    const types = ["completed", "wait", "active", "delayed", "failed"];
    const queue = await (0, queue_1.getQueue)();
    const grace = period && period.length ? (0, ms_1.default)(period) : void 0;
    if (!grace) {
        return (0, utils_1.throwYellow)("Incorrect period");
    }
    const status = options.status || "completed";
    if (!types.includes(status)) {
        return (0, utils_1.throwYellow)(`Unknown status, must be one of: ${types.join(", ")}`);
    }
    await (0, utils_1.answer)(exports.vorpal, "Clean", options.yes);
    const limit = Number.isInteger(options.limit)
        ? options.limit
        : 0;
    await queue.clean(grace, limit, status);
    (0, utils_1.logGreen)(`Jobs cleaned`);
}));
exports.vorpal
    .command("logs <jobId>", "Get logs of job")
    .option("-s, --start <start>", "Start of logs")
    .option("-e, --end <end>", "End of logs")
    .action((0, utils_1.wrapTryCatch)(async ({ jobId, options }) => {
    const queue = await (0, queue_1.getQueue)();
    const { logs, count } = (await queue.getJobLogs(jobId, options.start, options.end));
    console.log(`Count of job logs: ${count}`);
    if (logs.length) {
        console.log("Logs:");
        (0, utils_1.logArray)(logs);
    }
}));
exports.vorpal
    .command("log <jobId> <data>", "Add log to job")
    .action((0, utils_1.wrapTryCatch)(async function ({ jobId, data, options }) {
    await (0, queue_1.getQueue)();
    const job = await (0, utils_1.getJob)(jobId);
    await (0, utils_1.answer)(exports.vorpal, "Add log", options.yes);
    await job.log(data);
    (0, utils_1.logGreen)("Log added to job");
}));
exports.vorpal.command("events-on", "Turn on logging of queue events").action((0, utils_1.wrapTryCatch)(async function () {
    const queue = await (0, queue_1.getQueue)();
    (0, queue_1.listenQueueEvents)(queue);
    (0, utils_1.logGreen)(`Logging of queue events enabled`);
}));
exports.vorpal.command("events-off", "Turn off logging of queue events").action((0, utils_1.wrapTryCatch)(async function () {
    await (0, queue_1.getQueue)();
    (0, queue_1.unlistenQueueEvents)();
    (0, utils_1.logGreen)(`Logging of queue events disabled`);
}));
exports.vorpal.history("bull-repl-default");
exports.vorpal.delimiter("BULL-REPL> ").show();
const command = (0, utils_1.getBootCommand)();
if (command) {
    exports.vorpal.exec(command);
}
